@@ -50,11 +50,19 @@ numpydoc_xref_param_type = True
 numpydoc_class_members_toctree = False
 numpydoc_attributes_as_param_list = True
 numpydoc_xref_aliases = {
+    'BIDSPath': ':class:`BIDSPath <mne_bids.BIDSPath>`',
+    'path-like': ':term:`path-like <mne:path-like>`',
+    'array-like': ':term:`array-like <mne:array-like>`',
+    'int': ':class:`int <python:int>`',
+    'bool': ':class:`bool <python:bool>`',
+    'float': ':class:`float <python:float>`',
+    'list': ':class:`list <python:list>`',
+    'tuple': ':class:`tuple <python:tuple>`',
     'NibabelImageObject': 'nibabel.spatialimages.SpatialImage',
 }
 numpydoc_xref_ignore = {
     # words
-    'of',
+    'instance', 'instances', 'of'
 }
 
 
@@ -67,6 +75,9 @@ default_role = 'autolink'  # XXX silently allows bad syntax, someone should fix
 # https://sphinx-doc.org/en/master/usage/configuration.html?#options-for-the-linkcheck-builder
 linkcheck_retries = 2
 linkcheck_rate_limit_timeout = 15.0
+linkcheck_ignore = [
+    r'https://www.researchgate.net/profile/.*',
+]
 
 # The suffix(es) of source filenames.
 # You can specify multiple suffix as a list of string:
@@ -132,8 +143,10 @@ html_theme_options = {
 
 html_context = {
     'versions_dropdown': {
-        'dev': 'v0.9 (devel)',
-        'stable': 'v0.8 (stable)',
+        'dev': 'v0.11 (devel)',
+        'stable': 'v0.10 (stable)',
+        'v0.9': 'v0.9',
+        'v0.8': 'v0.8',
         'v0.7': 'v0.7',
         'v0.6': 'v0.6',
         'v0.5': 'v0.5',
