@@ -19,7 +19,7 @@ In a second step we will read the organized dataset using MNE-BIDS.
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #          Richard Höchenberger <richard.hoechenberger@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 # %%
 # First we import some basic Python libraries, followed by MNE-Python and its
@@ -82,6 +82,14 @@ bids_path = BIDSPath(subject='01', session='01',
 write_raw_bids(raw, bids_path, events_data=events_data,
                event_id=event_id, overwrite=True)
 
+# %%
+# Let's pause and check that the information that we've written out to the
+# sidecar files that describe our data is correct.
+
+# Get the sidecar ``.json`` file
+print(bids_path.copy().update(extension='.json').fpath.read_text(
+    encoding='utf-8-sig'))
+
 # %%
 # The sample MEG dataset comes with fine-calibration and crosstalk files that
 # are required when processing Elekta/Neuromag/MEGIN data using MaxFilter®.
