@@ -2,7 +2,7 @@
 # Authors: Teon L Brooks <teon.brooks@gmail.com>
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 import os.path as op
 from pathlib import Path
 from functools import partial
@@ -23,7 +23,7 @@ from mne.utils._testing import requires_module
 
 from mne_bids.commands import (mne_bids_raw_to_bids,
                                mne_bids_cp,
-                               mne_bids_mark_bad_channels,
+                               mne_bids_mark_channels,
                                mne_bids_calibration_to_bids,
                                mne_bids_crosstalk_to_bids,
                                mne_bids_count_events,
@@ -52,9 +52,9 @@ def check_usage(module, force_help=False):
         assert 'Usage: ' in out.stdout.getvalue()
 
 
-def test_raw_to_bids(tmpdir):
+def test_raw_to_bids(tmp_path):
     """Test mne_bids raw_to_bids."""
-    output_path = str(tmpdir)
+    output_path = str(tmp_path)
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
@@ -87,9 +87,9 @@ def test_raw_to_bids(tmpdir):
             mne_bids_cp.run()
 
 
-def test_cp(tmpdir):
+def test_cp(tmp_path):
     """Test mne_bids cp."""
-    output_path = str(tmpdir)
+    output_path = str(tmp_path)
     data_path = op.join(base_path, 'brainvision', 'tests', 'data')
     raw_fname = op.join(data_path, 'test.vhdr')
     outname = op.join(output_path, 'test2.vhdr')
@@ -107,14 +107,14 @@ def test_cp(tmpdir):
             mne_bids_cp.run()
 
 
-def test_mark_bad_chanels_single_file(tmpdir):
-    """Test mne_bids mark_bad_channels."""
+def test_mark_bad_chanels_single_file(tmp_path):
+    """Test mne_bids mark_channels."""
 
     # Check that help is printed
-    check_usage(mne_bids_mark_bad_channels)
+    check_usage(mne_bids_mark_channels)
 
     # Create test dataset.
-    output_path = str(tmpdir)
+    output_path = str(tmp_path)
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
@@ -139,19 +139,20 @@ def test_mark_bad_chanels_single_file(tmpdir):
 
     args = tuple(args)
     with ArgvSetter(args):
-        mne_bids_mark_bad_channels.run()
+        mne_bids_mark_channels.run()
 
     # Check the data was properly written
     with pytest.warns(RuntimeWarning, match='The unit for chann*'):
-        raw = read_raw_bids(bids_path=bids_path)
+        raw = read_raw_bids(bids_path=bids_path, verbose=False)
     assert set(old_bads + ch_names) == set(raw.info['bads'])
 
-    # Test resettig bad channels.
+    # Test resetting bad channels.
     args = ('--subject_id', subject_id, '--task', task,
             '--bids_root', output_path, '--type', datatype,
-            '--ch_name', '', '--overwrite')
+            '--status', 'good', '--ch_name', '')
     with ArgvSetter(args):
-        mne_bids_mark_bad_channels.run()
+        mne_bids_mark_channels.run()
+    print('Finished running the reset...')
 
     # Check the data was properly written
     with pytest.warns(RuntimeWarning, match='The unit for chann*'):
@@ -159,14 +160,14 @@ def test_mark_bad_chanels_single_file(tmpdir):
     assert raw.info['bads'] == []
 
 
-def test_mark_bad_chanels_multiple_files(tmpdir):
-    """Test mne_bids mark_bad_channels."""
+def test_mark_bad_chanels_multiple_files(tmp_path):
+    """Test mne_bids mark_channels."""
 
     # Check that help is printed
-    check_usage(mne_bids_mark_bad_channels)
+    check_usage(mne_bids_mark_channels)
 
     # Create test dataset.
-    output_path = str(tmpdir)
+    output_path = str(tmp_path)
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
@@ -191,7 +192,7 @@ def test_mark_bad_chanels_multiple_files(tmpdir):
 
     args = tuple(args)
     with ArgvSetter(args):
-        mne_bids_mark_bad_channels.run()
+        mne_bids_mark_channels.run()
 
     # Check the data was properly written
     for subject in subjects:
@@ -201,13 +202,13 @@ def test_mark_bad_chanels_multiple_files(tmpdir):
         assert set(old_bads + ch_names) == set(raw.info['bads'])
 
 
-def test_calibration_to_bids(tmpdir):
+def test_calibration_to_bids(tmp_path):
     """Test mne_bids calibration_to_bids."""
 
     # Check that help is printed
     check_usage(mne_bids_calibration_to_bids)
 
-    output_path = str(tmpdir)
+    output_path = str(tmp_path)
     data_path = Path(testing.data_path())
     fine_cal_fname = data_path / 'SSS' / 'sss_cal_mgh.dat'
     bids_path = BIDSPath(subject=subject_id, root=output_path)
@@ -221,13 +222,13 @@ def test_calibration_to_bids(tmpdir):
     assert bids_path.meg_calibration_fpath.exists()
 
 
-def test_crosstalk_to_bids(tmpdir):
+def test_crosstalk_to_bids(tmp_path):
     """Test mne_bids crosstalk_to_bids."""
 
     # Check that help is printed
     check_usage(mne_bids_crosstalk_to_bids)
 
-    output_path = str(tmpdir)
+    output_path = str(tmp_path)
     data_path = Path(testing.data_path())
     crosstalk_fname = data_path / 'SSS' / 'ct_sparse.fif'
     bids_path = BIDSPath(subject=subject_id, root=output_path)
@@ -242,14 +243,14 @@ def test_crosstalk_to_bids(tmpdir):
 
 
 @requires_pandas
-def test_count_events(tmpdir):
+def test_count_events(tmp_path):
     """Test mne_bids count_events."""
 
     # Check that help is printed
     check_usage(mne_bids_count_events)
 
     # Create test dataset.
-    output_path = str(tmpdir)
+    output_path = str(tmp_path)
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
@@ -260,7 +261,7 @@ def test_count_events(tmpdir):
     event_id = {'auditory/left': 1, 'auditory/right': 2, 'visual/left': 3,
                 'visual/right': 4, 'face': 5, 'button': 32}
 
-    bids_path = BIDSPath(subject='01', root=output_path)
+    bids_path = BIDSPath(subject='01', root=output_path, task='foo')
     write_raw_bids(raw, bids_path, events, event_id, overwrite=True,
                    verbose=False)
 
@@ -270,17 +271,31 @@ def test_count_events(tmpdir):
     with ArgvSetter(('--bids_root', output_path, '--describe')):
         mne_bids_count_events.run()
 
+    with ArgvSetter(('--bids_root', output_path, '--silent')):
+        mne_bids_count_events.run()
+
+    with ArgvSetter(('--bids_root', output_path,
+                     '--output', str(Path(output_path) / 'counts.csv'))):
+        mne_bids_count_events.run()
+
+    with ArgvSetter(
+        ('--bids_root', output_path,
+         '--output', str(Path(output_path) / 'counts.csv'),
+         '--overwrite')
+    ):
+        mne_bids_count_events.run()
+
 
 @requires_matplotlib
 @requires_version('mne', '0.22')
-def test_inspect(tmpdir):
+def test_inspect(tmp_path):
     """Test mne_bids inspect."""
 
     # Check that help is printed
     check_usage(mne_bids_inspect)
 
     # Create test dataset.
-    bids_root = str(tmpdir)
+    bids_root = str(tmp_path)
     data_path = testing.data_path()
     subject = '01'
     task = 'test'
@@ -303,5 +318,4 @@ def test_inspect(tmpdir):
         args = ('--bids_root', bids_root, '--h_freq', h_freq,
                 '--find_flat', 0)
         with ArgvSetter(args):
-            with pytest.warns(RuntimeWarning, match='The unit for chann*'):
-                mne_bids_inspect.run()
+            mne_bids_inspect.run()
