@@ -7,7 +7,7 @@ BIDS_VERSION = "1.6.0"
 
 DOI = """https://doi.org/10.21105/joss.01896"""
 
-EPHY_ALLOWED_DATATYPES = ['meg', 'eeg', 'ieeg']
+EPHY_ALLOWED_DATATYPES = ['meg', 'eeg', 'ieeg', 'nirs']
 
 ALLOWED_DATATYPES = EPHY_ALLOWED_DATATYPES + ['anat', 'beh']
 
@@ -48,9 +48,11 @@ meg_manufacturers = {
 
 eeg_manufacturers = {'.vhdr': 'BrainProducts', '.eeg': 'BrainProducts',
                      '.edf': 'n/a', '.EDF': 'n/a', '.bdf': 'Biosemi',
+                     '.BDF': 'Biosemi',
                      '.set': 'n/a', '.fdt': 'n/a',
                      '.lay': 'Persyst', '.dat': 'Persyst',
-                     '.EEG': 'Nihon Kohden'}
+                     '.EEG': 'Nihon Kohden',
+                     '.cnt': 'Neuroscan', '.CNT': 'Neuroscan'}
 
 ieeg_manufacturers = {'.vhdr': 'BrainProducts', '.eeg': 'BrainProducts',
                       '.edf': 'n/a', '.EDF': 'n/a', '.set': 'n/a',
@@ -58,6 +60,8 @@ ieeg_manufacturers = {'.vhdr': 'BrainProducts', '.eeg': 'BrainProducts',
                       '.lay': 'Persyst', '.dat': 'Persyst',
                       '.EEG': 'Nihon Kohden'}
 
+nirs_manufacturers = {'.snirf': 'SNIRF'}
+
 # file-extension map to mne-python readers
 reader = {'.con': io.read_raw_kit, '.sqd': io.read_raw_kit,
           '.fif': io.read_raw_fif, '.pdf': io.read_raw_bti,
@@ -65,7 +69,9 @@ reader = {'.con': io.read_raw_kit, '.sqd': io.read_raw_kit,
           '.edf': io.read_raw_edf, '.EDF': io.read_raw_edf,
           '.bdf': io.read_raw_bdf,
           '.set': io.read_raw_eeglab, '.lay': io.read_raw_persyst,
-          '.EEG': io.read_raw_nihon}
+          '.EEG': io.read_raw_nihon,
+          '.cnt': io.read_raw_cnt, '.CNT': io.read_raw_cnt,
+          '.snirf': io.read_raw_snirf}
 
 
 # Merge the manufacturer dictionaries in a python2 / python3 compatible way
@@ -76,6 +82,7 @@ MANUFACTURERS = dict()
 MANUFACTURERS.update(meg_manufacturers)
 MANUFACTURERS.update(eeg_manufacturers)
 MANUFACTURERS.update(ieeg_manufacturers)
+MANUFACTURERS.update(nirs_manufacturers)
 
 # List of synthetic channels by manufacturer that are to be excluded from the
 # channel list. Currently this is only for stimulus channels.
@@ -86,44 +93,53 @@ IGNORED_CHANNELS = {'KIT/Yokogawa': ['STI 014'],
 
 allowed_extensions_meg = ['.con', '.sqd', '.fif', '.pdf', '.ds']
 allowed_extensions_eeg = ['.vhdr',  # BrainVision, accompanied by .vmrk, .eeg
-                          '.edf', '.EDF',  # European Data Format
+                          '.edf',  # European Data Format
                           '.bdf',  # Biosemi
                           '.set',  # EEGLAB, potentially accompanied by .fdt
                           ]
 
 allowed_extensions_ieeg = ['.vhdr',  # BrainVision, accompanied by .vmrk, .eeg
-                           '.edf', '.EDF',  # European Data Format
+                           '.edf',  # European Data Format
                            '.set',  # EEGLAB, potentially accompanied by .fdt
                            '.mef',  # MEF: Multiscale Electrophysiology File
                            '.nwb',  # Neurodata without borders
                            ]
 
+allowed_extensions_nirs = ['.snirf',  # SNIRF
+                           ]
+
 # allowed extensions (data formats) in BIDS spec
-ALLOWED_DATATYPE_EXTENSIONS = {'meg': allowed_extensions_meg,
-                               'eeg': allowed_extensions_eeg,
-                               'ieeg': allowed_extensions_ieeg}
+ALLOWED_DATATYPE_EXTENSIONS = {
+    'meg': allowed_extensions_meg,
+    'eeg': allowed_extensions_eeg,
+    'ieeg': allowed_extensions_ieeg,
+    'nirs': allowed_extensions_nirs
+}
 
 # allow additional extensions that are not BIDS
 # compliant, but we will convert to the
 # recommended formats
 ALLOWED_INPUT_EXTENSIONS = \
     allowed_extensions_meg + allowed_extensions_eeg + \
-    allowed_extensions_ieeg + ['.lay', '.EEG']
+    allowed_extensions_ieeg + allowed_extensions_nirs + \
+    ['.lay', '.EEG', '.cnt', '.CNT']
 
 # allowed suffixes (i.e. last "_" delimiter in the BIDS filenames before
 # the extension)
 ALLOWED_FILENAME_SUFFIX = [
     'meg', 'markers', 'eeg', 'ieeg', 'T1w', 'FLASH',  # datatype
     'participants', 'scans',
-    'electrodes', 'channels', 'coordsystem', 'events',  # sidecars
+    'electrodes', 'optodes', 'channels', 'coordsystem', 'events',  # sidecars
     'headshape', 'digitizer',  # meg-specific sidecars
-    'beh', 'physio', 'stim'  # behavioral
+    'beh', 'physio', 'stim',  # behavioral
+    'nirs'
 ]
 
 # converts suffix to known path modalities
 SUFFIX_TO_DATATYPE = {
-    'meg': 'meg', 'eeg': 'eeg', 'ieeg': 'ieeg', 'T1w': 'anat',
-    'headshape': 'meg', 'digitizer': 'meg', 'markers': 'meg'
+    'meg': 'meg', 'headshape': 'meg', 'digitizer': 'meg', 'markers': 'meg',
+    'eeg': 'eeg', 'ieeg': 'ieeg',
+    'T1w': 'anat', 'FLASH': 'anat'
 }
 
 # allowed BIDS extensions (extension in the BIDS filename)
@@ -144,12 +160,12 @@ ALLOWED_PATH_ENTITIES_SHORT = {'sub': 'subject', 'ses': 'session',
                                'task': 'task', 'acq': 'acquisition',
                                'run': 'run', 'proc': 'processing',
                                'space': 'space', 'rec': 'recording',
-                               'split': 'split', 'suffix': 'suffix'}
+                               'split': 'split'}
 
 # Annotations to never remove during reading or writing
 ANNOTATIONS_TO_KEEP = ('BAD_ACQ_SKIP',)
 
-coordsys_standard_template = [
+BIDS_STANDARD_TEMPLATE_COORDINATE_SYSTEMS = [
     'ICBM452AirSpace',
     'ICBM452Warp5Space',
     'IXI549Space',
@@ -191,18 +207,27 @@ coordsys_standard_template_deprecated = [
     'UNCInfant2V23',
 ]
 
-coordsys_meg = ['CTF', 'ElektaNeuromag', '4DBti', 'KitYokogawa', 'ChietiItab']
-coordsys_eeg = ['CapTrak']
-coordsys_ieeg = ['Pixels', 'ACPC']
+# accepted BIDS formats, which may be subject to change
+# depending on the specification
+BIDS_IEEG_COORDINATE_FRAMES = ['ACPC', 'Pixels']
+BIDS_MEG_COORDINATE_FRAMES = ['CTF', 'ElektaNeuromag',
+                              '4DBti', 'KitYokogawa',
+                              'ChietiItab']
+BIDS_EEG_COORDINATE_FRAMES = ['CapTrak']
+
+# accepted coordinate SI units
+BIDS_COORDINATE_UNITS = ['m', 'cm', 'mm']
 coordsys_wildcard = ['Other']
-coordsys_shared = (coordsys_standard_template +
-                   coordsys_standard_template_deprecated +
-                   coordsys_wildcard)
+BIDS_SHARED_COORDINATE_FRAMES = (BIDS_STANDARD_TEMPLATE_COORDINATE_SYSTEMS +
+                                 coordsys_standard_template_deprecated +
+                                 coordsys_wildcard)
 
 ALLOWED_SPACES = dict()
-ALLOWED_SPACES['meg'] = coordsys_shared + coordsys_meg + coordsys_eeg
-ALLOWED_SPACES['eeg'] = coordsys_shared + coordsys_meg + coordsys_eeg
-ALLOWED_SPACES['ieeg'] = coordsys_shared + coordsys_ieeg
+ALLOWED_SPACES['meg'] = ALLOWED_SPACES['eeg'] = \
+    BIDS_SHARED_COORDINATE_FRAMES + BIDS_MEG_COORDINATE_FRAMES + \
+    BIDS_EEG_COORDINATE_FRAMES
+ALLOWED_SPACES['ieeg'] = \
+    BIDS_SHARED_COORDINATE_FRAMES + BIDS_IEEG_COORDINATE_FRAMES
 ALLOWED_SPACES['anat'] = None
 ALLOWED_SPACES['beh'] = None
 
@@ -221,17 +246,6 @@ ENTITY_VALUE_TYPE = {
     'extension': 'label'
 }
 
-# accepted BIDS formats, which may be subject to change
-# depending on the specification
-BIDS_IEEG_COORDINATE_FRAMES = ['ACPC', 'Pixels', 'Other']
-BIDS_MEG_COORDINATE_FRAMES = ['CTF', 'ElektaNeuromag',
-                              '4DBti', 'KitYokogawa',
-                              'ChietiItab', 'Other']
-BIDS_EEG_COORDINATE_FRAMES = ['CapTrak']
-
-# accepted coordinate SI units
-BIDS_COORDINATE_UNITS = ['m', 'cm', 'mm']
-
 # mapping from supported BIDs coordinate frames -> MNE
 BIDS_TO_MNE_FRAMES = {
     'CTF': 'ctf_head',
@@ -240,10 +254,9 @@ BIDS_TO_MNE_FRAMES = {
     'ElektaNeuromag': 'head',
     'ChietiItab': 'head',
     'CapTrak': 'head',
-    'ACPC': 'ras',  # XXX: there is no ACPC in mne-python and so this is a one-way mapping from BIDS -> MNE  # noqa
+    'ACPC': 'mri',  # assumes T1 is ACPC-aligned, if not the coordinates are lost  # noqa
     'fsaverage': 'mni_tal',  # XXX: note fsaverage and MNI305 are the same  # noqa
-    'MNI305': 'mni_tal',
-    'Other': 'unknown'
+    'MNI305': 'mni_tal'
 }
 
 # mapping from supported MNE coordinate frames -> BIDS
@@ -253,11 +266,7 @@ BIDS_TO_MNE_FRAMES = {
 MNE_TO_BIDS_FRAMES = {
     'ctf_head': 'CTF',
     'head': 'CapTrak',
-    'mni_tal': 'fsaverage',
-    # 'fs_tal': 'fsaverage',  # XXX: not used
-    'unknown': 'Other',
-    'ras': 'Other',
-    'mri': 'Other'
+    'mni_tal': 'fsaverage'
 }
 
 # these coordinate frames in mne-python are related to scalp/meg
@@ -279,6 +288,17 @@ MNE_FRAME_TO_STR = {val: key for key, val in MNE_STR_TO_FRAME.items()}
 
 # see BIDS specification for description we copied over from each
 BIDS_COORD_FRAME_DESCRIPTIONS = {
+    'acpc': 'The origin of the coordinate system is at the Anterior '
+            'Commissure and the negative y-axis is passing through the '
+            'Posterior Commissure. The positive z-axis is passing through '
+            'a mid-hemispheric point in the superior direction.',
+    'pixels': 'If electrodes are localized in 2D space (only x and y are '
+              'specified and z is n/a), then the positions in this file '
+              'must correspond to the locations expressed in pixels on '
+              'the photo/drawing/rendering of the electrodes on the brain. '
+              'In this case, coordinates must be (row,column) pairs, with '
+              '(0,0) corresponding to the upper left pixel and (N,0) '
+              'corresponding to the lower left pixel.',
     'ctf': 'ALS orientation and the origin between the ears',
     'elektaneuromag': 'RAS orientation and the origin between the ears',
     '4dbti': 'ALS orientation and the origin between the ears',
@@ -296,8 +316,62 @@ BIDS_COORD_FRAME_DESCRIPTIONS = {
     'fsaverage': 'Defined by FreeSurfer, the MRI (surface RAS) origin is '
                  'at the center of a 256×256×256 mm^3 anisotropic volume '
                  '(may not be in the center of the head).',
+    'icbm452airspace': 'Reference space defined by the "average of 452 '
+                       'T1-weighted MRIs of normal young adult brains" '
+                       'with "linear transforms of the subjects into the '
+                       'atlas space using a 12-parameter affine '
+                       'transformation"',
+    'icbm452warp5space': 'Reference space defined by the "average of 452 '
+                         'T1-weighted MRIs of normal young adult brains" '
+                         '"based on a 5th order polynomial transformation '
+                         'into the atlas space"',
+    'ixi549space': 'Reference space defined by the average of the "549 (...) '
+                   'subjects from the IXI dataset" linearly transformed to '
+                   'ICBM MNI 452.',
+    'fsaveragesym': 'The fsaverage is a dual template providing both '
+                    'volumetric and surface coordinates references. The '
+                    'volumetric template corresponds to a FreeSurfer variant '
+                    'of MNI305 space. The fsaverageSym atlas also defines a '
+                    'symmetric surface reference system (formerly described '
+                    'as fsaveragesym).',
+    'fslr': 'The fsLR is a dual template providing both volumetric and '
+            'surface coordinates references. The volumetric template '
+            'corresponds to MNI152NLin6Asym. Surface templates are given '
+            'at several sampling densities: 164k (used by HCP pipelines '
+            'for 3T and 7T anatomical analysis), 59k (used by HCP pipelines '
+            'for 7T MRI bold and DWI analysis), 32k (used by HCP pipelines '
+            'for 3T MRI bold and DWI analysis), or 4k (used by HCP '
+            'pipelines for MEG analysis) fsaverage_LR surface '
+            'reconstructed from the T1w image.',
+    'mnicolin27': 'Average of 27 T1 scans of a single subject.',
+    'mni152lin': 'Also known as ICBM (version with linear coregistration).',
+    'mni152nlin6sym': 'Also known as symmetric ICBM 6th generation '
+                      '(non-linear coregistration).',
+    'mni152nlin6asym': 'A variation of MNI152NLin6Sym built by A. Janke that '
+                       'is released as the MNI template of FSL. Volumetric '
+                       'templates included with HCP-Pipelines correspond to '
+                       'this template too.',
+    'mni305': 'Also known as avg305.',
+    'nihpd': 'Pediatric templates generated from the NIHPD sample. Available '
+             'for different age groups (4.5–18.5 y.o., 4.5–8.5 y.o., '
+             '7–11 y.o., 7.5–13.5 y.o., 10–14 y.o., 13–18.5 y.o. This '
+             'template also comes in either -symmetric or -asymmetric flavor.',
+    'oasis30antsoasisants':
+        'See https://figshare.com/articles/ANTs_ANTsR_Brain_Templates/915436',
+    'oasis30atropos':
+        'See https://mindboggle.info/data.html',
+    'talairach': 'Piecewise linear scaling of the brain is implemented as '
+                 'described in TT88.',
+    'uncinfant': 'Infant Brain Atlases from Neonates to 1- and 2-year-olds.'
 }
 
+for letter in ('a', 'b', 'c'):
+    for sym in ('Sym', 'Asym'):
+        BIDS_COORD_FRAME_DESCRIPTIONS[f'mni152nlin2009{letter}{sym}'] = \
+            'Also known as ICBM (non-linear coregistration with 40 iterations,'
+        ' released in 2009). It comes in either three different flavours '
+        'each in symmetric or asymmetric version.'
+
 REFERENCES = {'mne-bids':
               'Appelhoff, S., Sanderson, M., Brooks, T., Vliet, M., '
               'Quentin, R., Holdgraf, C., Chaumon, M., Mikulan, E., '
@@ -326,7 +400,9 @@ REFERENCES = {'mne-bids':
               'D\'Ambrosio, S., David, O., … Hermes, D. (2019). iEEG-BIDS, '
               'extending the Brain Imaging Data Structure specification '
               'to human intracranial electrophysiology. Scientific Data, '
-              '6, 102. https://doi.org/10.1038/s41597-019-0105-7'}
+              '6, 102. https://doi.org/10.1038/s41597-019-0105-7',
+              'nirs':
+              'In preperation'}
 
 
 # Mapping subject information between MNE-BIDS and MNE-Python.
@@ -373,3 +449,91 @@ def _map_options(what, key, fro, to):
                            "accepted. Use 'mne', or 'bids'.".format(fro, to))
 
     return mapped_option
+
+
+# Which JSON data can safely be transferred from a non-anonymized to an
+# anonymized dataset without accidentally exposing personal identifiable
+# information
+ANONYMIZED_JSON_KEY_WHITELIST = [
+    # Common
+    'Manufacturer',
+    'ManufacturersModelName',
+    'InstitutionName',
+    'InstitutionalDepartmentName',
+    'InstitutionAddress',
+    'DeviceSerialNumber',
+    # MRI
+    # Many of these are not standardized, but produced by dcm2niix.
+    'Modality',
+    'MagneticFieldStrength',
+    'ImagingFrequency',
+    'StationName',
+    'SeriesInstanceUID',
+    'StudyInstanceUID',
+    'StudyID',
+    'BodyPartExamined',
+    'PatientPosition',
+    'ProcedureStepDescription',
+    'SoftwareVersions',
+    'MRAcquisitionType',
+    'SeriesDescription',
+    'ProtocolName',
+    'ScanningSequence',
+    'SequenceVariant',
+    'ScanOptions',
+    'SequenceName',
+    'ImageType',
+    'SeriesNumber',
+    'AcquisitionNumber',
+    'SliceThickness',
+    'SAR',
+    'EchoTime',
+    'RepetitionTime',
+    'InversionTime',
+    'FlipAngle',
+    'PartialFourier',
+    'BaseResolution',
+    'ShimSetting',
+    'TxRefAmp',
+    'PhaseResolution',
+    'ReceiveCoilName',
+    'ReceiveCoilActiveElements',
+    'PulseSequenceDetails',
+    'ConsistencyInfo',
+    'PercentPhaseFOV',
+    'PercentSampling',
+    'PhaseEncodingSteps',
+    'AcquisitionMatrixPE',
+    'PixelBandwidth',
+    'DwellTime',
+    'ImageOrientationPatientDICOM',
+    'InPlanePhaseEncodingDirectionDICOM',
+    'ConversionSoftware',
+    'ConversionSoftwareVersion',
+    # Electrophys common
+    'TaskName',
+    'TaskDescription',
+    'Instructions',
+    'PowerLineFrequency',
+    'SamplingFrequency',
+    'SoftwareFilters',
+    'RecordingType',
+    'EEGChannelCount',
+    'EOGChannelCount',
+    'ECGChannelCount',
+    'EMGChannelCount',
+    'MiscChannelCount',
+    'TriggerChannelCount',
+    'RecordingDuration',
+    # EEG
+    'EEGReference',
+    'EEGPlacementScheme',
+    # MEG
+    'DewarPosition',
+    'DigitizedLandmarks',
+    'DigitizedHeadPoints',
+    'MEGChannelCount',
+    'MEGREFChannelCount',
+    'ContinuousHeadLocalization',
+    'HeadCoilFrequency'
+]
