@@ -2,22 +2,31 @@
 # Authors: Richard Höchenberger <richard.hoechenberger@gmail.com>
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from pathlib import Path
 
 import numpy as np
 
 import mne
-from mne.preprocessing import annotate_flat
-from mne.utils import logger
 
-from mne_bids import read_raw_bids, mark_bad_channels
+from mne.utils import logger, verbose
+from mne.fixes import _compare_version
+
+if _compare_version(mne.__version__, '<', '1.0.dev0'):  # pragma: no cover
+    from mne.preprocessing import annotate_flat
+    _annotate_flat_func = annotate_flat
+else:
+    from mne.preprocessing import annotate_amplitude
+    _annotate_flat_func = annotate_amplitude
+
+from mne_bids import read_raw_bids, mark_channels
 from mne_bids.read import _from_tsv, _read_events
 from mne_bids.write import _events_tsv
 from mne_bids.config import ALLOWED_DATATYPE_EXTENSIONS
 
 
+@verbose
 def inspect_dataset(bids_path, find_flat=True, l_freq=None, h_freq=None,
                     show_annotations=True, verbose=None):
     """Inspect and annotate BIDS raw data.
@@ -39,7 +48,7 @@ def inspect_dataset(bids_path, find_flat=True, l_freq=None, h_freq=None,
 
     Parameters
     ----------
-    bids_path : mne_bids.BIDSPath
+    bids_path : BIDSPath
         A :class:`mne_bids.BIDSPath` containing at least a ``root``. All
         matching files will be inspected. To select only a subset of the data,
         set more :class:`mne_bids.BIDSPath` attributes. If ``datatype`` is not
@@ -51,17 +60,19 @@ def inspect_dataset(bids_path, find_flat=True, l_freq=None, h_freq=None,
     find_flat : bool
         Whether to auto-detect channels producing "flat" signals, i.e., with
         unusually low variability. Flat **segments** will be added to
-        ``*_events.tsv``, while channels with more than 5% of flat data will be
-        marked as ``bad`` in ``*_channels.tsv``.
+        ``*_events.tsv``, while channels with more than 5 percent of flat data
+        will be marked as ``bad`` in ``*_channels.tsv``.
 
         .. note::
-            This function calls :func:`mne.preprocessing.annotate_flat` and
-            will only consider segments of at least **50 ms consecutive
+            This function calls ``mne.preprocessing.annotate_amplitude``
+            (MNE-Python 1.0 or newer) or ``mne.preprocessing.annotate_flat``
+            (older versions of MNE-Python)
+            and will only consider segments of at least **50 ms consecutive
             flatness** as "flat" (deviating from MNE-Python's default of 5 ms).
-            If more than 5% of a channel's data has been marked as flat, the
-            entire channel will be added to the list of bad channels. Only flat
-            time segments applying to channels **not** marked as bad will be
-            added to ``*_events.tsv``.
+            If more than 5 percent of a channel's data has been marked as flat,
+            the entire channel will be added to the list of bad channels. Only
+            flat time segments applying to channels **not** marked as bad will
+            be added to ``*_events.tsv``.
 
     l_freq : float | None
         The high-pass filter cutoff frequency to apply when displaying the
@@ -75,9 +86,7 @@ def inspect_dataset(bids_path, find_flat=True, l_freq=None, h_freq=None,
         Whether to show annotations (events, bad segments, …) or not. If
         ``False``, toggling annotations mode by pressing ``A`` will be disabled
         as well.
-    verbose : bool | None
-        If a boolean, whether or not to produce verbose output. If ``None``,
-        use the default log level.
+    %(verbose)s
 
     Examples
     --------
@@ -102,8 +111,7 @@ def inspect_dataset(bids_path, find_flat=True, l_freq=None, h_freq=None,
 
     for bids_path_ in bids_paths:
         _inspect_raw(bids_path=bids_path_, l_freq=l_freq, h_freq=h_freq,
-                     find_flat=find_flat, show_annotations=show_annotations,
-                     verbose=verbose)
+                     find_flat=find_flat, show_annotations=show_annotations)
 
 
 # XXX This this should probably be refactored into a class attribute someday.
@@ -112,8 +120,7 @@ _global_vars = dict(raw_fig=None,
                     mne_close_key=None)
 
 
-def _inspect_raw(*, bids_path, l_freq, h_freq, find_flat, show_annotations,
-                 verbose=None):
+def _inspect_raw(*, bids_path, l_freq, h_freq, find_flat, show_annotations):
     """Raw data inspection."""
     # Delay the import
     import matplotlib
@@ -128,8 +135,18 @@ def _inspect_raw(*, bids_path, l_freq, h_freq, find_flat, show_annotations,
 
     if find_flat:
         raw.load_data()  # Speeds up processing dramatically
-        flat_annot, flat_chans = annotate_flat(raw=raw, min_duration=0.05,
-                                               verbose=verbose)
+        if _annotate_flat_func.__name__ == 'annotate_amplitude':
+            flat_annot, flat_chans = annotate_amplitude(
+                raw=raw,
+                flat=0,
+                min_duration=0.05,
+                bad_percent=5
+            )
+        else:  # pragma: no cover
+            flat_annot, flat_chans = annotate_flat(
+                raw=raw,
+                min_duration=0.05
+            )
         new_annot = raw.annotations + flat_annot
         raw.set_annotations(new_annot)
         raw.info['bads'] = list(set(raw.info['bads'] + flat_chans))
@@ -163,8 +180,7 @@ def _inspect_raw(*, bids_path, l_freq, h_freq, find_flat, show_annotations,
                              flat_chans=flat_chans,
                              old_annotations=old_annotations,
                              new_annotations=new_annotations,
-                             bids_path=bids_path,
-                             verbose=verbose)
+                             bids_path=bids_path)
         _global_vars['raw_fig'] = None
 
     def _keypress_callback(event):
@@ -208,7 +224,7 @@ def _annotations_almost_equal(old_annotations, new_annotations):
         return False
 
 
-def _save_annotations(*, annotations, bids_path, verbose):
+def _save_annotations(*, annotations, bids_path):
     # Attach the new Annotations to our raw data so we can easily convert them
     # to events, which will be stored in the *_events.tsv sidecar.
     extra_params = dict()
@@ -219,8 +235,7 @@ def _save_annotations(*, annotations, bids_path, verbose):
                         verbose='warning')
     raw.set_annotations(annotations)
     events, durs, descrs = _read_events(events_data=None, event_id=None,
-                                        task=bids_path.task, raw=raw,
-                                        verbose=False)
+                                        bids_path=bids_path, raw=raw)
 
     # Write sidecar – or remove it if no events are left.
     events_tsv_fname = (bids_path.copy()
@@ -231,7 +246,7 @@ def _save_annotations(*, annotations, bids_path, verbose):
     if len(events) > 0:
         _events_tsv(events=events, durations=durs, raw=raw,
                     fname=events_tsv_fname, trial_type=descrs,
-                    overwrite=True, verbose=verbose)
+                    overwrite=True)
     elif events_tsv_fname.exists():
         logger.info(f'No events remaining after interactive inspection, '
                     f'removing {events_tsv_fname.name}')
@@ -240,7 +255,7 @@ def _save_annotations(*, annotations, bids_path, verbose):
 
 def _save_raw_if_changed(*, old_bads, new_bads, flat_chans,
                          old_annotations, new_annotations,
-                         bids_path, verbose):
+                         bids_path):
     """Save bad channel selection if it has been changed.
 
     Parameters
@@ -302,11 +317,10 @@ def _save_raw_if_changed(*, old_bads, new_bads, flat_chans,
     return _save_raw_dialog_box(bads=bads,
                                 bad_descriptions=bad_descriptions,
                                 annotations=annotations,
-                                bids_path=bids_path, verbose=verbose)
+                                bids_path=bids_path)
 
 
-def _save_raw_dialog_box(*, bads, bad_descriptions, annotations, bids_path,
-                         verbose):
+def _save_raw_dialog_box(*, bads, bad_descriptions, annotations, bids_path):
     """Display a dialog box asking whether to save the changes."""
     # Delay the imports
     import matplotlib
@@ -374,10 +388,9 @@ def _save_raw_dialog_box(*, bads, bad_descriptions, annotations, bids_path,
 
         if bads is not None:
             _save_bads(bads=bads, descriptions=bad_descriptions,
-                       bids_path=bids_path, verbose=verbose)
+                       bids_path=bids_path)
         if annotations is not None:
-            _save_annotations(annotations=annotations, bids_path=bids_path,
-                              verbose=verbose)
+            _save_annotations(annotations=annotations, bids_path=bids_path)
 
     def _dont_save_callback(event):
         plt.close(event.canvas.figure)  # Close dialog
@@ -401,7 +414,7 @@ def _save_raw_dialog_box(*, bads, bad_descriptions, annotations, bids_path,
     _global_vars['dialog_fig'] = fig
 
 
-def _save_bads(*, bads, descriptions, bids_path, verbose):
+def _save_bads(*, bads, descriptions, bids_path):
     """Update the set of channels marked as bad.
 
     Parameters
@@ -411,7 +424,7 @@ def _save_bads(*, bads, descriptions, bids_path, verbose):
     descriptions : list
         The values to be written to the `status_description` column.
     """
-    # We pass overwrite=True, causing all channels not passed as bad here to
-    # be marked as good.
-    mark_bad_channels(ch_names=bads, descriptions=descriptions,
-                      bids_path=bids_path, overwrite=True, verbose=verbose)
+    # We first make all channels not passed as bad here to be marked as good.
+    mark_channels(bids_path=bids_path, ch_names=[], status='good')
+    mark_channels(bids_path=bids_path, ch_names=bads, status='bad',
+                  descriptions=descriptions)
