@@ -4,19 +4,27 @@
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #          mne-bids developers
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import json
 from collections import OrderedDict
 
-from mne.channels import DigMontage
-from mne.utils import logger, _validate_type
+import numpy as np
+
+from mne.channels import DigMontage, make_dig_montage
+from mne.utils import (
+    logger, _validate_type, verbose, _check_on_missing, _on_missing
+)
+from mne.io import read_fiducials
+from mne.io.constants import FIFF
+
 from mne_bids import BIDSPath
 from mne_bids.utils import _write_json
 
 
 # TODO: add support for tsv files
-def update_sidecar_json(bids_path, entries, verbose=True):
+@verbose
+def update_sidecar_json(bids_path, entries, verbose=None):
     """Update sidecar files using a dictionary or JSON file.
 
     Will update metadata fields inside the path defined by
@@ -34,7 +42,7 @@ def update_sidecar_json(bids_path, entries, verbose=True):
 
     Parameters
     ----------
-    bids_path : mne_bids.BIDSPath
+    bids_path : BIDSPath
         The set of paths to update. The :class:`mne_bids.BIDSPath` instance
         passed here **must** have the ``.root`` attribute set. The
         ``.datatype`` attribute **may** be set. If ``.datatype`` is
@@ -45,8 +53,7 @@ def update_sidecar_json(bids_path, entries, verbose=True):
     entries : dict | str | pathlib.Path
         A dictionary, or JSON file that defines the
         sidecar fields and corresponding values to be updated to.
-    verbose : bool
-        The verbosity level.
+    %(verbose)s
 
     Notes
     -----
@@ -104,9 +111,8 @@ def update_sidecar_json(bids_path, entries, verbose=True):
             sidecar_tmp = json.load(
                 tmp_f, object_pairs_hook=OrderedDict)
 
-    if verbose:
-        logger.debug(sidecar_tmp)
-        logger.debug(f'Updating {fpath}...')
+    logger.debug(sidecar_tmp)
+    logger.debug(f'Updating {fpath}...')
 
     # load in sidecar filepath
     with open(fpath, 'r') as tmp_f:
@@ -117,7 +123,7 @@ def update_sidecar_json(bids_path, entries, verbose=True):
     sidecar_json.update(**sidecar_tmp)
 
     # write back the sidecar JSON
-    _write_json(fpath, sidecar_json, overwrite=True, verbose=verbose)
+    _write_json(fpath, sidecar_json, overwrite=True)
 
 
 def _update_sidecar(sidecar_fname, key, val):
@@ -135,10 +141,14 @@ def _update_sidecar(sidecar_fname, key, val):
     with open(sidecar_fname, 'r', encoding='utf-8-sig') as fin:
         sidecar_json = json.load(fin)
     sidecar_json[key] = val
-    _write_json(sidecar_fname, sidecar_json, overwrite=True, verbose=False)
+    _write_json(sidecar_fname, sidecar_json, overwrite=True)
 
 
-def update_anat_landmarks(bids_path, landmarks):
+@verbose
+def update_anat_landmarks(
+    bids_path, landmarks, *, fs_subject=None, fs_subjects_dir=None,
+    kind=None, on_missing='raise', verbose=None
+):
     """Update the anatomical landmark coordinates of an MRI scan.
 
     This will change the ``AnatomicalLandmarkCoordinates`` entry in the
@@ -146,23 +156,58 @@ def update_anat_landmarks(bids_path, landmarks):
 
     Parameters
     ----------
-    bids_path : mne_bids.BIDSPath
+    bids_path : BIDSPath
         Path of the MR image.
-    landmarks : mne.channels.DigMontage
+    landmarks : mne.channels.DigMontage | path-like
         An :class:`mne.channels.DigMontage` instance with coordinates for the
         nasion and left and right pre-auricular points in MRI voxel
-        coordinates.
+        coordinates. Alternatively, the path to a ``*-fiducials.fif`` file as
+        produced by the MNE-Python coregistration GUI or via
+        :func:`mne.io.write_fiducials`.
 
         .. note:: :func:`mne_bids.get_anat_landmarks` provides a convenient and
                   reliable way to generate the landmark coordinates in the
                   required coordinate system.
 
+        .. note:: If ``path-like``, ``fs_subject`` and ``fs_subjects_dir``
+                  must be provided as well.
+
+        .. versionchanged:: 0.10
+           Added support for ``path-like`` input.
+    fs_subject : str | None
+        The subject identifier used for FreeSurfer. Must be provided if
+        ``landmarks`` is ``path-like``; otherwise, it will be ignored.
+    fs_subjects_dir : path-like | None
+        The FreeSurfer subjects directory. If ``None``, defaults to the
+        ``SUBJECTS_DIR`` environment variable. Must be provided if
+        ``landmarks`` is ``path-like``; otherwise, it will be ignored.
+    kind : str | None
+        The suffix of the anatomical landmark names in the JSON sidecar.
+        A suffix might be present e.g. to distinguish landmarks between
+        sessions. If provided, should not include a leading underscore ``_``.
+        For example, if the landmark names in the JSON sidecar file are
+        ``LPA_ses-1``, ``RPA_ses-1``, ``NAS_ses-1``, you should pass
+        ``'ses-1'`` here.
+        If ``None``, no suffix is appended, the landmarks named
+        ``Nasion`` (or ``NAS``), ``LPA``, and ``RPA`` will be used.
+
+        .. versionadded:: 0.10
+    on_missing : 'ignore' | 'warn' | 'raise'
+        How to behave if the specified landmarks cannot be found in the MRI
+        JSON sidecar file.
+
+        .. versionadded:: 0.10
+    %(verbose)s
+
     Notes
     -----
     .. versionadded:: 0.8
     """
     _validate_type(item=bids_path, types=BIDSPath, item_name='bids_path')
-    _validate_type(item=landmarks, types=DigMontage, item_name='landmarks')
+    _validate_type(
+        item=landmarks, types=(DigMontage, 'path-like'), item_name='landmarks'
+    )
+    _check_on_missing(on_missing)
 
     # Do some path verifications and fill in some gaps the users might have
     # left (datatype and extension)
@@ -209,6 +254,19 @@ def update_anat_landmarks(bids_path, landmarks):
             f'bids_path. Tried the following filenames: '
             f'{", ".join([p.name for p in tried_paths])}')
 
+    if not isinstance(landmarks, DigMontage):  # it's pathlike
+        if fs_subject is None:
+            raise ValueError(
+                'You must provide the "fs_subject" parameter when passing the '
+                'path to fiducials'
+            )
+        landmarks = _get_landmarks_from_fiducials_file(
+            bids_path=bids_path,
+            fname=landmarks,
+            fs_subject=fs_subject,
+            fs_subjects_dir=fs_subjects_dir
+        )
+
     positions = landmarks.get_positions()
     coord_frame = positions['coord_frame']
     if coord_frame != 'mri_voxel':
@@ -230,7 +288,10 @@ def update_anat_landmarks(bids_path, landmarks):
         if coords is None:
             missing_points.append(name)
         else:
-            name_to_coords_map[name] = list(coords)
+            # Funnily, np.float64 is JSON-serializabe, while np.float32 is not!
+            # Thus, cast to float64 to avoid issues (which e.g. may arise when
+            # fiducials were read from disk!)
+            name_to_coords_map[name] = list(coords.astype('float64'))
 
     if missing_points:
         raise ValueError(
@@ -239,12 +300,76 @@ def update_anat_landmarks(bids_path, landmarks):
             f'following points are missing: '
             f'{", ".join(missing_points)}')
 
-    mri_json = {
-        'AnatomicalLandmarkCoordinates': name_to_coords_map
-    }
-
     bids_path_json = bids_path.copy().update(extension='.json')
     if not bids_path_json.fpath.exists():  # Must exist before we can update it
         _write_json(bids_path_json.fpath, dict())
 
+    mri_json = json.loads(bids_path_json.fpath.read_text(encoding='utf-8'))
+    if 'AnatomicalLandmarkCoordinates' not in mri_json:
+        _on_missing(
+            on_missing=on_missing,
+            msg=f'No AnatomicalLandmarkCoordinates section found in '
+                f'{bids_path_json.fpath.name}',
+            error_klass=KeyError
+        )
+        mri_json['AnatomicalLandmarkCoordinates'] = dict()
+
+    for name, coords in name_to_coords_map.items():
+        if kind is not None:
+            name = f'{name}_{kind}'
+
+        if name not in mri_json['AnatomicalLandmarkCoordinates']:
+            _on_missing(
+                on_missing=on_missing,
+                msg=f'Anatomical landmark not found in '
+                    f'{bids_path_json.fpath.name}: {name}',
+                error_klass=KeyError
+            )
+
+        mri_json['AnatomicalLandmarkCoordinates'][name] = coords
+
     update_sidecar_json(bids_path=bids_path_json, entries=mri_json)
+
+
+def _get_landmarks_from_fiducials_file(*, bids_path, fname, fs_subject,
+                                       fs_subjects_dir):
+    """Get anatomical landmarks from fiducials file, in MRI voxel space."""
+    # avoid dicrular imports
+    from mne_bids.write import (
+        _get_t1w_mgh, _mri_landmarks_to_mri_voxels, _get_fid_coords
+    )
+
+    digpoints, coord_frame = read_fiducials(fname)
+
+    # All of this should be guaranteed, but better be safe than sorry!
+    assert coord_frame == FIFF.FIFFV_COORD_MRI
+    assert digpoints[0]['ident'] == FIFF.FIFFV_POINT_LPA
+    assert digpoints[1]['ident'] == FIFF.FIFFV_POINT_NASION
+    assert digpoints[2]['ident'] == FIFF.FIFFV_POINT_RPA
+
+    montage_loaded = make_dig_montage(
+        lpa=digpoints[0]['r'],
+        nasion=digpoints[1]['r'],
+        rpa=digpoints[2]['r'],
+        coord_frame='mri'
+    )
+    landmark_coords_mri, _ = _get_fid_coords(dig_points=montage_loaded.dig)
+    landmark_coords_mri = np.asarray(
+        (landmark_coords_mri['lpa'],
+         landmark_coords_mri['nasion'],
+         landmark_coords_mri['rpa'])
+    )
+
+    t1w_mgh = _get_t1w_mgh(fs_subject, fs_subjects_dir)
+    landmark_coords_voxels = _mri_landmarks_to_mri_voxels(
+        mri_landmarks=landmark_coords_mri * 1000,  # in mm
+        t1_mgh=t1w_mgh
+    )
+    montage_voxels = make_dig_montage(
+        lpa=landmark_coords_voxels[0],
+        nasion=landmark_coords_voxels[1],
+        rpa=landmark_coords_voxels[2],
+        coord_frame='mri_voxel'
+    )
+
+    return montage_voxels
