@@ -1,25 +1,17 @@
 """Test for the MNE BIDS path functions."""
 # Authors: Adam Li <adam2392@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 import os
 import os.path as op
 import shutil as sh
-
-# This is here to handle mne-python <0.20
-import warnings
 from pathlib import Path
 import shutil
 from datetime import datetime, timezone
 
 import pytest
 
-with warnings.catch_warnings():
-    warnings.filterwarnings(action='ignore',
-                            message="can't resolve package",
-                            category=ImportWarning)
-    import mne
-
+import mne
 from mne.datasets import testing
 from mne.io import anonymize_info
 
@@ -28,7 +20,8 @@ from mne_bids import (get_datatypes, get_entity_vals, print_dir_tree,
                       write_meg_calibration, write_meg_crosstalk)
 from mne_bids.path import (_parse_ext, get_entities_from_fname,
                            _find_best_candidates, _find_matching_sidecar,
-                           _filter_fnames)
+                           _filter_fnames, search_folder_for_text,
+                           get_bids_path_from_fname)
 from mne_bids.config import ALLOWED_PATH_ENTITIES_SHORT
 
 from test_read import _read_raw_fif, warning_str
@@ -40,15 +33,15 @@ run = '01'
 acq = None
 task = 'testing'
 
-bids_path = BIDSPath(
+_bids_path = BIDSPath(
     subject=subject_id, session=session_id, run=run, acquisition=acq,
     task=task)
 
 
 @pytest.fixture(scope='session')
-def return_bids_test_dir(tmpdir_factory):
+def return_bids_test_dir(tmp_path_factory):
     """Return path to a written test BIDS dir."""
-    bids_root = str(tmpdir_factory.mktemp('mnebids_utils_test_bids_ds'))
+    bids_root = str(tmp_path_factory.mktemp('mnebids_utils_test_bids_ds'))
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
@@ -67,7 +60,7 @@ def return_bids_test_dir(tmpdir_factory):
     events = mne.read_events(events_fname)
     events = events[events[:, 2] != 0]
 
-    bids_path.update(root=bids_root)
+    bids_path = _bids_path.copy().update(root=bids_root)
     # Write multiple runs for test_purposes
     for run_idx in [run, '02']:
         name = bids_path.copy().update(run=run_idx)
@@ -105,6 +98,13 @@ def test_get_keys(return_bids_test_dir):
 def test_get_entity_vals(entity, expected_vals, kwargs, return_bids_test_dir):
     """Test getting a list of entities."""
     bids_root = return_bids_test_dir
+    # Add some derivative data that should be ignored by get_entity_vals()
+    deriv_path = Path(bids_root) / 'derivatives'
+    deriv_meg_dir = deriv_path / 'pipeline' / 'sub-deriv' / 'ses-deriv' / 'meg'
+    deriv_meg_dir.mkdir(parents=True)
+    (deriv_meg_dir / 'sub-deriv_ses-deriv_task-deriv_meg.fif').touch()
+    (deriv_meg_dir / 'sub-deriv_ses-deriv_task-deriv_meg.json').touch()
+
     if kwargs is None:
         kwargs = dict()
 
@@ -125,6 +125,43 @@ def test_get_entity_vals(entity, expected_vals, kwargs, return_bids_test_dir):
         assert entities == [f'{entity_long_to_short[entity]}-{val}'
                             for val in expected_vals]
 
+        # Test without ignoring the derivatives dir
+        entities = get_entity_vals(
+            root=bids_root, entity_key=entity, **kwargs, ignore_dirs=None
+        )
+        if entity not in ('acquisition', 'run'):
+            assert 'deriv' in entities
+
+    # Clean up
+    shutil.rmtree(deriv_path)
+
+
+def test_search_folder_for_text(capsys):
+    """Test finding entries."""
+    with pytest.raises(ValueError, match='is not a directory'):
+        search_folder_for_text('foo', 'i_dont_exist')
+
+    # We check the testing directory
+    test_dir = op.dirname(__file__)
+    search_folder_for_text('n/a', test_dir)
+    captured = capsys.readouterr()
+    assert 'sub-01_ses-eeg_electrodes.tsv' in captured.out
+    assert '    1    name      x         y         z         impedance' in \
+        captured.out.split('\n')
+    assert '    66   ECG       n/a       n/a       n/a       n/a' in \
+        captured.out.split('\n')
+
+    # test if pathlib.Path object
+    search_folder_for_text('n/a', Path(test_dir))
+
+    # test returning a string and without line numbers
+    out = search_folder_for_text(
+        'n/a', test_dir, line_numbers=False, return_str=True)
+    assert 'sub-01_ses-eeg_electrodes.tsv' in out
+    assert '    name      x         y         z         impedance' in \
+        out.split('\n')
+    assert '    ECG       n/a       n/a       n/a       n/a' in out.split('\n')
+
 
 def test_print_dir_tree(capsys):
     """Test printing a dir tree."""
@@ -167,33 +204,30 @@ def test_print_dir_tree(capsys):
     assert '|--- __pycache__{}'.format(os.sep) in out.split('\n')
     assert '.pyc' not in out
 
-    # errors if return_str not a bool
-    with pytest.raises(ValueError, match='must be either True or False.'):
-        print_dir_tree(test_dir, return_str='yes')
 
-
-def test_make_folders(tmpdir):
+def test_make_folders(tmp_path):
     """Test that folders are created and named properly."""
     # Make sure folders are created properly
     bids_path = BIDSPath(subject='01', session='foo',
-                         datatype='eeg', root=str(tmpdir))
+                         datatype='eeg', root=str(tmp_path))
     bids_path.mkdir().directory
-    assert op.isdir(tmpdir / 'sub-01' / 'ses-foo' / 'eeg')
+    assert op.isdir(tmp_path / 'sub-01' / 'ses-foo' / 'eeg')
 
     # If we remove a kwarg the folder shouldn't be created
     bids_path = BIDSPath(subject='02', datatype='eeg',
-                         root=tmpdir)
+                         root=tmp_path)
     bids_path.mkdir().directory
-    assert op.isdir(tmpdir / 'sub-02' / 'eeg')
+    assert op.isdir(tmp_path / 'sub-02' / 'eeg')
 
     # Check if a pathlib.Path bids_root works.
     bids_path = BIDSPath(subject='03', session='foo',
-                         datatype='eeg', root=Path(tmpdir))
+                         datatype='eeg', root=tmp_path)
     bids_path.mkdir().directory
-    assert op.isdir(tmpdir / 'sub-03' / 'ses-foo' / 'eeg')
+    assert op.isdir(tmp_path / 'sub-03' / 'ses-foo' / 'eeg')
 
     # Check if bids_root=None creates folders in the current working directory
-    bids_root = tmpdir.mkdir("tmp")
+    bids_root = tmp_path / "tmp"
+    bids_root.mkdir()
     curr_dir = os.getcwd()
     os.chdir(bids_root)
     bids_path = BIDSPath(subject='04', session='foo',
@@ -223,6 +257,29 @@ def test_parse_ext():
     assert ext == '.nii.gz'
 
 
+@pytest.mark.parametrize('fname', [
+    'sub-01_ses-02_task-test_run-3_split-01_meg.fif',
+    'sub-01_ses-02_task-test_run-3_split-01',
+    ('/bids_root/sub-01/ses-02/meg/' +
+     'sub-01_ses-02_task-test_run-3_split-01_meg.fif'),
+    ('sub-01/ses-02/meg/' +
+     'sub-01_ses-02_task-test_run-3_split-01_meg.fif')
+])
+def test_get_bids_path_from_fname(fname):
+    bids_path = get_bids_path_from_fname(fname)
+    assert bids_path.basename == Path(fname).name
+
+    if '/bids_root/' in fname:
+        assert Path(bids_path.root) == Path('/bids_root')
+    else:
+        if 'meg' in fname:
+            # directory should match
+            assert Path(bids_path.directory) == Path('sub-01/ses-02/meg')
+
+        # root should be default '.'
+        assert str(bids_path.root) == '.'
+
+
 @pytest.mark.parametrize('fname', [
     'sub-01_ses-02_task-test_run-3_split-01_meg.fif',
     'sub-01_ses-02_task-test_run-3_split-01.fif',
@@ -233,17 +290,14 @@ def test_parse_ext():
 def test_get_entities_from_fname(fname):
     """Test parsing entities from a bids filename."""
     params = get_entities_from_fname(fname)
-    print(params)
     assert params['subject'] == '01'
     assert params['session'] == '02'
     assert params['run'] == '3'
     assert params['task'] == 'test'
     assert params['split'] == '01'
-    if 'meg' in fname:
-        assert params['suffix'] == 'meg'
     assert list(params.keys()) == ['subject', 'session', 'task',
                                    'acquisition', 'run', 'processing',
-                                   'space', 'recording', 'split', 'suffix']
+                                   'space', 'recording', 'split']
 
 
 @pytest.mark.parametrize('fname', [
@@ -269,15 +323,13 @@ def test_get_entities_from_fname_errors(fname):
 
     expected_keys = ['subject', 'session', 'task',
                      'acquisition', 'run', 'processing',
-                     'space', 'recording', 'split', 'suffix']
+                     'space', 'recording', 'split']
 
     assert params['subject'] == '01'
     assert params['session'] == '02'
     assert params['run'] == '3'
     assert params['task'] == 'test'
     assert params['split'] == '01'
-    if 'meg' in fname:
-        assert params['suffix'] == 'meg'
     if 'desc' in fname:
         assert params['desc'] == 'tfr'
         expected_keys.append('desc')
@@ -306,14 +358,14 @@ def test_find_best_candidates(candidate_list, best_candidates):
     assert _find_best_candidates(params, candidate_list) == best_candidates
 
 
-def test_find_matching_sidecar(return_bids_test_dir):
+def test_find_matching_sidecar(return_bids_test_dir, tmp_path):
     """Test finding a sidecar file from a BIDS dir."""
     bids_root = return_bids_test_dir
 
-    bids_fpath = bids_path.copy().update(root=bids_root)
+    bids_path = _bids_path.copy().update(root=bids_root)
 
     # Now find a sidecar
-    sidecar_fname = _find_matching_sidecar(bids_fpath,
+    sidecar_fname = _find_matching_sidecar(bids_path,
                                            suffix='coordsystem',
                                            extension='.json')
     expected_file = op.join('sub-01', 'ses-01', 'meg',
@@ -325,33 +377,71 @@ def test_find_matching_sidecar(return_bids_test_dir):
         open(sidecar_fname.replace('coordsystem.json',
                                    '2coordsystem.json'), 'w').close()
         print_dir_tree(bids_root)
-        _find_matching_sidecar(bids_fpath,
+        _find_matching_sidecar(bids_path,
                                suffix='coordsystem', extension='.json')
 
     # Find nothing and raise.
     with pytest.raises(RuntimeError, match='Did not find any'):
-        fname = _find_matching_sidecar(bids_fpath, suffix='foo',
+        fname = _find_matching_sidecar(bids_path, suffix='foo',
                                        extension='.bogus')
 
     # Find nothing and receive None and a warning.
     on_error = 'warn'
     with pytest.warns(RuntimeWarning, match='Did not find any'):
-        fname = _find_matching_sidecar(bids_fpath, suffix='foo',
+        fname = _find_matching_sidecar(bids_path, suffix='foo',
                                        extension='.bogus', on_error=on_error)
     assert fname is None
 
     # Find nothing and receive None.
     on_error = 'ignore'
-    fname = _find_matching_sidecar(bids_fpath, suffix='foo',
+    fname = _find_matching_sidecar(bids_path, suffix='foo',
                                    extension='.bogus', on_error=on_error)
     assert fname is None
 
     # Invalid on_error.
     on_error = 'hello'
     with pytest.raises(ValueError, match='Acceptable values for on_error are'):
-        _find_matching_sidecar(bids_fpath, suffix='coordsystem',
+        _find_matching_sidecar(bids_path, suffix='coordsystem',
                                extension='.json', on_error=on_error)
 
+    # Test behavior of suffix and extension params when suffix and extension
+    # are also (not) present in the passed BIDSPath
+    bids_path = BIDSPath(
+        subject='test', task='task', datatype='eeg', root=tmp_path
+    )
+    bids_path.mkdir()
+
+    for suffix, extension in zip(
+        ['eeg', 'eeg', 'events', 'events'],
+        ['.fif', '.json', '.tsv', '.json']
+    ):
+        bids_path.suffix = suffix
+        bids_path.extension = extension
+        bids_path.fpath.touch()
+
+    # suffix parameter should always override BIDSPath.suffix
+    bids_path.extension = '.json'
+
+    for bp_suffix in (None, 'eeg'):
+        bids_path.suffix = bp_suffix
+        s = _find_matching_sidecar(bids_path=bids_path, suffix='events')
+        assert Path(s).name == 'sub-test_task-task_events.json'
+
+    # extension parameter should always override BIDSPath.extension
+    bids_path.suffix = 'events'
+
+    for bp_extension in (None, '.json'):
+        bids_path.extension = bp_extension
+        s = _find_matching_sidecar(bids_path=bids_path, extension='.tsv')
+        assert Path(s).name == 'sub-test_task-task_events.tsv'
+
+    # If suffix and extension parameters are not passed, use BIDSPath
+    # attributes
+    bids_path.suffix = 'events'
+    bids_path.extension = '.tsv'
+    s = _find_matching_sidecar(bids_path=bids_path)
+    assert Path(s).name == 'sub-test_task-task_events.tsv'
+
 
 def test_bids_path_inference(return_bids_test_dir):
     """Test usage of BIDSPath object and fpath."""
@@ -365,16 +455,6 @@ def test_bids_path_inference(return_bids_test_dir):
     with pytest.raises(RuntimeError, match='Found more than one'):
         bids_path.fpath
 
-    # can't locate a file, but the basename should work
-    bids_path = BIDSPath(
-        subject=subject_id, session=session_id, acquisition=acq,
-        task=task, run='10', root=bids_root)
-    with pytest.warns(RuntimeWarning, match='Could not locate'):
-        fpath = bids_path.fpath
-        assert str(fpath) == op.join(bids_root, f'sub-{subject_id}',
-                                     f'ses-{session_id}',
-                                     bids_path.basename)
-
     # shouldn't error out when there is no uncertainty
     channels_fname = BIDSPath(subject=subject_id, session=session_id,
                               run=run, acquisition=acq, task=task,
@@ -694,7 +774,7 @@ def test_match(return_bids_test_dir):
     paths = bids_path_01.match()
     assert len(paths) == 0
 
-    bids_path_01 = bids_path.copy().update(root=None)
+    bids_path_01 = _bids_path.copy().update(root=None)
     with pytest.raises(RuntimeError, match='Cannot match'):
         bids_path_01.match()
 
@@ -753,9 +833,11 @@ def test_match(return_bids_test_dir):
     assert Path(paths[0]).parent.name == 'meg'
 
     # Test `check` parameter
-    bids_path_01 = bids_path.copy()
-    bids_path_01.update(session=None, task=None, run=None,
-                        suffix='foo', extension='.eeg', check=False)
+    bids_path_01 = _bids_path.copy()
+    bids_path_01.update(
+        root=bids_root, session=None, task=None, run=None,
+        suffix='foo', extension='.eeg', check=False
+    )
     bids_path_01.fpath.touch()
 
     assert bids_path_01.match(check=True) == []
@@ -764,19 +846,21 @@ def test_match(return_bids_test_dir):
 
 @pytest.mark.filterwarnings(warning_str['meas_date_set_to_none'])
 @pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
-def test_find_empty_room(return_bids_test_dir, tmpdir):
+def test_find_empty_room(return_bids_test_dir, tmp_path):
     """Test reading of empty room data."""
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
-    bids_root = tmpdir.mkdir("bids")
-    tmp_dir = tmpdir.mkdir("tmp")
+    bids_root = tmp_path / "bids"
+    bids_root.mkdir()
+    tmp_dir = tmp_path / "tmp"
+    tmp_dir.mkdir()
 
     raw = _read_raw_fif(raw_fname)
     bids_path = BIDSPath(subject='01', session='01',
                          task='audiovisual', run='01',
                          root=bids_root, suffix='meg')
-    write_raw_bids(raw, bids_path, overwrite=True)
+    write_raw_bids(raw, bids_path, overwrite=True, verbose=False)
 
     # No empty-room data present.
     er_basename = bids_path.find_empty_room()
@@ -800,7 +884,7 @@ def test_find_empty_room(return_bids_test_dir, tmpdir):
     er_bids_path = BIDSPath(subject='emptyroom', task='noise',
                             session=er_date, suffix='meg',
                             root=bids_root)
-    write_raw_bids(er_raw, er_bids_path, overwrite=True)
+    write_raw_bids(er_raw, er_bids_path, overwrite=True, verbose=False)
 
     recovered_er_bids_path = bids_path.find_empty_room()
     assert er_bids_path == recovered_er_bids_path
@@ -813,7 +897,7 @@ def test_find_empty_room(return_bids_test_dir, tmpdir):
         er_meas_date = datetime.strptime(date, '%Y%m%d')
         er_meas_date = er_meas_date.replace(tzinfo=timezone.utc)
         er_raw.set_meas_date(er_meas_date)
-        write_raw_bids(er_raw, er_bids_path)
+        write_raw_bids(er_raw, er_bids_path, verbose=False)
 
     best_er_basename = bids_path.find_empty_room()
     assert best_er_basename.session == '20021204'
@@ -833,7 +917,8 @@ def test_find_empty_room(return_bids_test_dir, tmpdir):
 
     # test that the `AssociatedEmptyRoom` key in MEG sidecar is respected
 
-    bids_root = tmpdir.mkdir('associated-empty-room')
+    bids_root = tmp_path / 'associated-empty-room'
+    bids_root.mkdir()
     raw = _read_raw_fif(raw_fname)
     meas_date = datetime(year=2020, month=1, day=10, tzinfo=timezone.utc)
     er_date = datetime(year=2010, month=1, day=1, tzinfo=timezone.utc)
@@ -876,16 +961,20 @@ def test_find_empty_room(return_bids_test_dir, tmpdir):
     write_raw_bids(raw, bids_path=bids_path, empty_room=None, overwrite=True)
     assert bids_path.find_empty_room() == er_matching_date_bids_path
 
+    # If we enforce searching only via `AssociatedEmptyRoom`, we should get no
+    # result
+    assert bids_path.find_empty_room(use_sidecar_only=True) is None
+
 
 @pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
-def test_find_emptyroom_ties(tmpdir):
+def test_find_emptyroom_ties(tmp_path):
     """Test that we receive a warning on a date tie."""
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
 
-    bids_root = str(tmpdir)
-    bids_path.update(root=bids_root)
+    bids_root = str(tmp_path)
+    bids_path = _bids_path.copy().update(root=bids_root)
     session = '20010101'
     er_dir_path = BIDSPath(subject='emptyroom', session=session,
                            datatype='meg', root=bids_root)
@@ -916,14 +1005,14 @@ def test_find_emptyroom_ties(tmpdir):
 
 
 @pytest.mark.filterwarnings(warning_str['channel_unit_changed'])
-def test_find_emptyroom_no_meas_date(tmpdir):
+def test_find_emptyroom_no_meas_date(tmp_path):
     """Test that we warn if measurement date can be read or inferred."""
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
 
-    bids_root = str(tmpdir)
-    bids_path.update(root=bids_root)
+    bids_root = str(tmp_path)
+    bids_path = _bids_path.copy().update(root=bids_root)
     er_session = 'mysession'
     er_meas_date = None
 
@@ -969,28 +1058,28 @@ def test_meg_calibration_fpath(return_bids_test_dir):
 
     # File exists, so BIDSPath.meg_calibration_fpath should return a non-None
     # value.
-    bids_path_ = bids_path.copy().update(subject='01', root=bids_root)
+    bids_path_ = _bids_path.copy().update(subject='01', root=bids_root)
     assert bids_path_.meg_calibration_fpath is not None
 
     # subject not set.
-    bids_path_ = bids_path.copy().update(root=bids_root, subject=None)
+    bids_path_ = _bids_path.copy().update(root=bids_root, subject=None)
     with pytest.raises(ValueError, match='root and subject must be set'):
         bids_path_.meg_calibration_fpath
 
     # root not set.
-    bids_path_ = bids_path.copy().update(subject='01', root=None)
+    bids_path_ = _bids_path.copy().update(subject='01', root=None)
     with pytest.raises(ValueError, match='root and subject must be set'):
         bids_path_.meg_calibration_fpath
 
     # datatype is not 'meg''.
-    bids_path_ = bids_path.copy().update(subject='01', root=bids_root,
-                                         datatype='eeg')
+    bids_path_ = _bids_path.copy().update(subject='01', root=bids_root,
+                                          datatype='eeg')
     with pytest.raises(ValueError, match='Can only find .* for MEG'):
         bids_path_.meg_calibration_fpath
 
     # Delete the fine-calibration file. BIDSPath.meg_calibration_fpath
     # should then return None.
-    bids_path_ = bids_path.copy().update(subject='01', root=bids_root)
+    bids_path_ = _bids_path.copy().update(subject='01', root=bids_root)
     Path(bids_path_.meg_calibration_fpath).unlink()
     assert bids_path_.meg_calibration_fpath is None
 
@@ -1000,30 +1089,30 @@ def test_meg_crosstalk_fpath(return_bids_test_dir):
 
     # File exists, so BIDSPath.crosstalk_fpath should return a non-None
     # value.
-    bids_path_ = bids_path.copy().update(subject='01', root=bids_root)
-    assert bids_path_.meg_crosstalk_fpath is not None
+    bids_path = _bids_path.copy().update(subject='01', root=bids_root)
+    assert bids_path.meg_crosstalk_fpath is not None
 
     # subject not set.
-    bids_path_ = bids_path.copy().update(root=bids_root, subject=None)
+    bids_path = _bids_path.copy().update(root=bids_root, subject=None)
     with pytest.raises(ValueError, match='root and subject must be set'):
-        bids_path_.meg_crosstalk_fpath
+        bids_path.meg_crosstalk_fpath
 
     # root not set.
-    bids_path_ = bids_path.copy().update(subject='01', root=None)
+    bids_path = _bids_path.copy().update(subject='01', root=None)
     with pytest.raises(ValueError, match='root and subject must be set'):
-        bids_path_.meg_crosstalk_fpath
+        bids_path.meg_crosstalk_fpath
 
     # datatype is not 'meg''.
-    bids_path_ = bids_path.copy().update(subject='01', root=bids_root,
+    bids_path = _bids_path.copy().update(subject='01', root=bids_root,
                                          datatype='eeg')
     with pytest.raises(ValueError, match='Can only find .* for MEG'):
-        bids_path_.meg_crosstalk_fpath
+        bids_path.meg_crosstalk_fpath
 
     # Delete the crosstalk file. BIDSPath.meg_crosstalk_fpath should then
     # return None.
-    bids_path_ = bids_path.copy().update(subject='01', root=bids_root)
-    Path(bids_path_.meg_crosstalk_fpath).unlink()
-    assert bids_path_.meg_crosstalk_fpath is None
+    bids_path = _bids_path.copy().update(subject='01', root=bids_root)
+    Path(bids_path.meg_crosstalk_fpath).unlink()
+    assert bids_path.meg_crosstalk_fpath is None
 
 
 def test_datasetdescription_with_bidspath(return_bids_test_dir):
@@ -1037,11 +1126,20 @@ def test_datasetdescription_with_bidspath(return_bids_test_dir):
         root=return_bids_test_dir, suffix='dataset_description',
         extension='.json', check=False)
     assert bids_path.fpath.as_posix() == \
-           Path(f'{return_bids_test_dir}/dataset_description.json').as_posix()
+        Path(f'{return_bids_test_dir}/dataset_description.json').as_posix()
 
     # setting it via update should work
     bids_path = BIDSPath(root=return_bids_test_dir,
                          extension='.json', check=True)
     bids_path.update(suffix='dataset_description', check=False)
     assert bids_path.fpath.as_posix() == \
-           Path(f'{return_bids_test_dir}/dataset_description.json').as_posix()
+        Path(f'{return_bids_test_dir}/dataset_description.json').as_posix()
+
+
+def test_update_fail_check_no_change():
+    bids_path = BIDSPath(subject='test')
+    try:
+        bids_path.update(suffix='ave')
+    except Exception:
+        pass
+    assert bids_path.suffix is None
