@@ -1,7 +1,7 @@
 """Test for the MNE BIDS updating of BIDS datasets."""
 # Authors: Adam Li <adam2392@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 import json
 import os.path as op
 from pathlib import Path
@@ -33,9 +33,9 @@ bids_path = BIDSPath(
 
 
 @pytest.fixture(scope='session')
-def _get_bids_test_dir(tmpdir_factory):
+def _get_bids_test_dir(tmp_path_factory):
     """Return path to a written test BIDS dir."""
-    bids_root = str(tmpdir_factory.mktemp('mnebids_utils_test_bids_ds'))
+    bids_root = str(tmp_path_factory.mktemp('mnebids_utils_test_bids_ds'))
     data_path = testing.data_path()
     raw_fname = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc_raw.fif')
@@ -138,7 +138,7 @@ def test_update_sidecar_jsons(_get_bids_test_dir, _bids_validate,
 
 
 @requires_nibabel()
-def test_update_anat_landmarks(tmpdir):
+def test_update_anat_landmarks(tmp_path):
     """Test updating the anatomical landmarks of an MRI scan."""
     data_path = Path(testing.data_path())
     raw_path = data_path / 'MEG' / 'sample' / 'sample_audvis_trunc_raw.fif'
@@ -146,7 +146,7 @@ def test_update_anat_landmarks(tmpdir):
     t1_path = data_path / 'subjects' / 'sample' / 'mri' / 'T1.mgz'
     fs_subject = 'sample'
     fs_subjects_dir = data_path / 'subjects'
-    bids_root = Path(tmpdir)
+    bids_root = tmp_path
     bids_path_mri = BIDSPath(subject=subject_id, session=session_id,
                              acquisition=acq, root=bids_root, datatype='anat',
                              suffix='T1w')
@@ -177,12 +177,30 @@ def test_update_anat_landmarks(tmpdir):
     )
 
     # Remove JSON sidecar; updating the anatomical landmarks should re-create
-    # the file
+    # the file unless `on_missing` is `'raise'`
     bids_path_mri_json.fpath.unlink()
-    update_anat_landmarks(bids_path=bids_path_mri, landmarks=landmarks_new)
+    with pytest.raises(
+        KeyError,
+        match='No AnatomicalLandmarkCoordinates section found'
+    ):
+        update_anat_landmarks(bids_path=bids_path_mri, landmarks=landmarks_new)
 
-    with bids_path_mri_json.fpath.open(encoding='utf-8') as f:
-        mri_json = json.load(f)
+    update_anat_landmarks(
+        bids_path=bids_path_mri, landmarks=landmarks_new, on_missing='ignore'
+    )
+
+    with pytest.raises(KeyError, match='landmark not found'):
+        update_anat_landmarks(
+            bids_path=bids_path_mri, landmarks=landmarks_new, kind='ses-1'
+        )
+    update_anat_landmarks(
+        bids_path=bids_path_mri, landmarks=landmarks_new, kind='ses-1',
+        on_missing='ignore'
+    )
+
+    mri_json = json.loads(bids_path_mri_json.fpath.read_text(encoding='utf-8'))
+    assert 'NAS' in mri_json['AnatomicalLandmarkCoordinates']
+    assert 'NAS_ses-1' in mri_json['AnatomicalLandmarkCoordinates']
 
     assert np.allclose(
         landmarks_new.dig[1]['r'],
@@ -233,3 +251,30 @@ def test_update_anat_landmarks(tmpdir):
                        match='did not contain all required cardinal points'):
         update_anat_landmarks(bids_path=bids_path_mri,
                               landmarks=landmarks_invalid)
+
+    # Test with path-like landmarks
+    fiducials_path = (data_path / 'subjects' / 'sample' / 'bem' /
+                      'sample-fiducials.fif')
+
+    update_anat_landmarks(
+        bids_path=bids_path_mri,
+        landmarks=fiducials_path,
+        fs_subject='sample',
+        fs_subjects_dir=data_path / 'subjects'
+    )
+    expected_coords_in_voxels = np.array(
+        [[68.38202,  45.24057,  43.439808],  # noqa: E241
+         [42.27006,  30.758774, 74.09837 ],  # noqa: E202, E241
+         [17.044853, 46.586075, 42.618504]]
+    )
+    mri_json = json.loads(
+        bids_path_mri_json.fpath.read_text(encoding='utf-8')
+    )
+    for landmark, expected_coords in zip(
+        ('LPA', 'NAS', 'RPA'),
+        expected_coords_in_voxels
+    ):
+        assert np.allclose(
+            mri_json['AnatomicalLandmarkCoordinates'][landmark],
+            expected_coords
+        )
