@@ -6,7 +6,7 @@
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #          Matt Sanderson <matt.sanderson@mq.edu.au>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 import json
 import os
 import re
@@ -15,12 +15,11 @@ from os import path as op
 
 import numpy as np
 from mne.channels import make_standard_montage
-from mne.io.constants import FIFF
 from mne.io.kit.kit import get_kit_info
 from mne.io.pick import pick_types
-from mne.utils import warn, logger
+from mne.utils import warn, logger, verbose, check_version
 
-from mne_bids.tsv_handler import _to_tsv, _tsv_to_str
+from mne_bids.tsv_handler import _to_tsv
 
 
 # This regex matches key-val pairs. Any characters are allowed in the key and
@@ -68,6 +67,7 @@ def _get_ch_type_mapping(fro='mne', to='bids'):
         mapping = dict(eeg='EEG', misc='MISC', stim='TRIG', emg='EMG',
                        ecog='ECOG', seeg='SEEG', eog='EOG', ecg='ECG',
                        resp='RESP', bio='MISC', dbs='DBS',
+                       fnirs_cw_amplitude='NIRSCWAMPLITUDE',
                        # MEG channels
                        meggradaxial='MEGGRADAXIAL', megmag='MEGMAG',
                        megrefgradaxial='MEGREFGRADAXIAL',
@@ -77,7 +77,7 @@ def _get_ch_type_mapping(fro='mne', to='bids'):
     elif fro == 'bids' and to == 'mne':
         mapping = dict(EEG='eeg', MISC='misc', TRIG='stim', EMG='emg',
                        ECOG='ecog', SEEG='seeg', EOG='eog', ECG='ecg',
-                       RESP='resp',
+                       RESP='resp', NIRS='fnirs_cw_amplitude',
                        # No MEG channels for now
                        # Many to one mapping
                        VEOG='eog', HEOG='eog', DBS='dbs')
@@ -89,7 +89,7 @@ def _get_ch_type_mapping(fro='mne', to='bids'):
     return mapping
 
 
-def _handle_datatype(raw, datatype, verbose=True):
+def _handle_datatype(raw, datatype):
     """Check if datatype exists in raw object or infer datatype if possible.
 
     Parameters
@@ -101,9 +101,6 @@ def _handle_datatype(raw, datatype, verbose=True):
         `mne.utils._handle_datatype()` will attempt to infer the datatype from
         the ``raw`` object. In case of multiple data types in the ``raw``
         object, ``datatype`` must not be ``None``.
-    verbose : bool
-        If ``True``, this will print additional information in the case of
-        combined MEG and iEEG/EEG recordings.
 
     Returns
     -------
@@ -114,14 +111,12 @@ def _handle_datatype(raw, datatype, verbose=True):
         _check_datatype(raw, datatype)
         # MEG data is not supported by BrainVision or EDF files
         if datatype in ['eeg', 'ieeg'] and 'meg' in raw:
-            if verbose:
-                print(os.linesep + f"Both {datatype} and 'meg' data found. "
-                                   f"BrainVision and EDF do not support 'meg' "
-                                   f"data. The data will therefore be stored "
-                                   f"as 'meg' data. If you wish to store your "
-                                   f"{datatype} data in BrainVision or EDF, "
-                                   f"please remove the 'meg' channels from "
-                                   f"your recording." + os.linesep)
+            logger.info(f"{os.linesep}Both {datatype} and 'meg' data found. "
+                        f"BrainVision and EDF do not support 'meg' data. "
+                        f"The data will therefore be stored as 'meg' data. "
+                        f"If you wish to store your {datatype} data in "
+                        f"BrainVision or EDF, please remove the 'meg'"
+                        f"channels from your recording.{os.linesep}")
             datatype = 'meg'
     else:
         datatypes = list()
@@ -132,6 +127,12 @@ def _handle_datatype(raw, datatype, verbose=True):
             datatypes.append('meg')
         if 'eeg' in raw:
             datatypes.append('eeg')
+        if 'fnirs_cw_amplitude' in raw:
+            if not check_version('mne', '1.0'):  # pragma: no cover
+                raise RuntimeError(
+                    'fNIRS support in MNE-BIDS requires MNE-Python version 1.0'
+                )
+            datatypes.append('nirs')
         if len(datatypes) == 0:
             raise ValueError('No MEG, EEG or iEEG channels found in data. '
                              'Please use raw.set_channel_types to set the '
@@ -184,7 +185,7 @@ def _check_types(variables):
                              f"expected.")
 
 
-def _write_json(fname, dictionary, overwrite=False, verbose=False):
+def _write_json(fname, dictionary, overwrite=False):
     """Write JSON to a file."""
     if op.exists(fname) and not overwrite:
         raise FileExistsError(f'"{fname}" already exists. '
@@ -195,24 +196,21 @@ def _write_json(fname, dictionary, overwrite=False, verbose=False):
         fid.write(json_output)
         fid.write('\n')
 
-    if verbose is True:
-        print(os.linesep + f"Writing '{fname}'..." + os.linesep)
-        print(json_output)
+    logger.info(f"Writing '{fname}'...")
 
 
-def _write_tsv(fname, dictionary, overwrite=False, verbose=False):
+@verbose
+def _write_tsv(fname, dictionary, overwrite=False, verbose=None):
     """Write an ordered dictionary to a .tsv file."""
     if op.exists(fname) and not overwrite:
         raise FileExistsError(f'"{fname}" already exists. '
                               'Please set overwrite to True.')
     _to_tsv(dictionary, fname)
 
-    if verbose:
-        print(os.linesep + f"Writing '{fname}'..." + os.linesep)
-        print(_tsv_to_str(dictionary))
+    logger.info(f"Writing '{fname}'...")
 
 
-def _write_text(fname, text, overwrite=False, verbose=True):
+def _write_text(fname, text, overwrite=False):
     """Write text to a file."""
     if op.exists(fname) and not overwrite:
         raise FileExistsError(f'"{fname}" already exists. '
@@ -221,9 +219,7 @@ def _write_text(fname, text, overwrite=False, verbose=True):
         fid.write(text)
         fid.write('\n')
 
-    if verbose:
-        print(os.linesep + f"Writing '{fname}'..." + os.linesep)
-        print(text)
+    logger.info(f"Writing '{fname}'...")
 
 
 def _check_key_val(key, val):
@@ -282,21 +278,6 @@ def _infer_eeg_placement_scheme(raw):
     return placement_scheme
 
 
-def _extract_landmarks(dig):
-    """Extract NAS, LPA, and RPA from raw.info['dig']."""
-    coords = dict()
-    landmarks = {d['ident']: d for d in dig
-                 if d['kind'] == FIFF.FIFFV_POINT_CARDINAL}
-    if landmarks:
-        if FIFF.FIFFV_POINT_NASION in landmarks:
-            coords['NAS'] = landmarks[FIFF.FIFFV_POINT_NASION]['r'].tolist()
-        if FIFF.FIFFV_POINT_LPA in landmarks:
-            coords['LPA'] = landmarks[FIFF.FIFFV_POINT_LPA]['r'].tolist()
-        if FIFF.FIFFV_POINT_RPA in landmarks:
-            coords['RPA'] = landmarks[FIFF.FIFFV_POINT_RPA]['r'].tolist()
-    return coords
-
-
 def _scale_coord_to_meters(coord, unit):
     """Scale units to meters (mne-python default)."""
     if unit == 'cm':
@@ -342,7 +323,9 @@ def _check_anonymize(anonymize, raw, ext):
                              'is able to store in FIF format, must '
                              f'be less than {daysback_max}')
     keep_his = anonymize['keep_his'] if 'keep_his' in anonymize else False
-    return daysback, keep_his
+    keep_source = anonymize['keep_source'] if 'keep_source' in \
+        anonymize else False
+    return daysback, keep_his, keep_source
 
 
 def _get_anonymization_daysback(raw):
@@ -367,7 +350,8 @@ def _get_anonymization_daysback(raw):
     return daysback_min, daysback_max
 
 
-def get_anonymization_daysback(raws):
+@verbose
+def get_anonymization_daysback(raws, verbose=None):
     """Get the group min and max number of daysback necessary for BIDS specs.
 
     .. warning:: It is important that you remember the anonymization
@@ -386,6 +370,7 @@ def get_anonymization_daysback(raws):
     ----------
     raw : mne.io.Raw | list of mne.io.Raw
         Subject raw data or list of raw data from several subjects.
+    %(verbose)s
 
     Returns
     -------
@@ -404,14 +389,14 @@ def get_anonymization_daysback(raws):
             daysback_min_list.append(daysback_min)
             daysback_max_list.append(daysback_max)
     if not daysback_min_list or not daysback_max_list:
-        raise ValueError('All measurement dates are None, ' +
+        raise ValueError('All measurement dates are None, '
                          'pass any `daysback` value to anonymize.')
     daysback_min = max(daysback_min_list)
     daysback_max = min(daysback_max_list)
     if daysback_min > daysback_max:
-        raise ValueError('The dataset spans more time than can be ' +
-                         'accomodated by MNE, you may have to ' +
-                         'not follow BIDS recommendations and use' +
+        raise ValueError('The dataset spans more time than can be '
+                         'accomodated by MNE, you may have to '
+                         'not follow BIDS recommendations and use'
                          'anonymized dates after 1925')
     return daysback_min, daysback_max
 
@@ -445,7 +430,7 @@ def _check_datatype(raw, datatype):
     -------
     None
     """
-    supported_types = ('meg', 'eeg', 'ieeg')
+    supported_types = ('meg', 'eeg', 'ieeg', 'nirs')
     if datatype not in supported_types:
         raise ValueError(
             f'The specified datatype {datatype} is currently not supported. '
@@ -457,6 +442,8 @@ def _check_datatype(raw, datatype):
         datatype_matches = True
     elif datatype == 'meg' and datatype in raw:
         datatype_matches = True
+    elif datatype == 'nirs' and 'fnirs_cw_amplitude' in raw:
+        datatype_matches = True
     elif datatype == 'ieeg':
         ieeg_types = ('seeg', 'ecog', 'dbs')
         if any(ieeg_type in raw for ieeg_type in ieeg_types):
